@@ -10,6 +10,7 @@ import yaml
 
 from ploomber.spec.DAGSpec import DAGSpec
 from ploomber.env.EnvDict import EnvDict
+from ploomber.util.util import load_dotted_path
 
 
 def process_arg(s):
@@ -120,17 +121,6 @@ def _add_args_from_env_dict(parser, env_dict):
         parser.add_argument('--env__' + arg, help='Default: {}'.format(val))
 
 
-def _parse_module(s):
-    parts = s.split('.')
-
-    if len(parts) < 2:
-        raise ImportError('Invalid module name, must be a dot separated '
-                          'string, with at least '
-                          '[module_name].[function_name]')
-
-    return '.'.join(parts[:-1]), parts[-1]
-
-
 def _parse_signature_from_callable(callable_):
     """
     Parse a callable signature, return a dictionary with
@@ -147,7 +137,7 @@ def _parse_signature_from_callable(callable_):
         for k, v in sig.parameters.items() if v.default != inspect._empty
     }
 
-    return required, defaults
+    return required, defaults, sig.parameters
 
 
 def get_desc(doc, arg):
@@ -155,6 +145,20 @@ def get_desc(doc, arg):
     return None if arg_data is None else arg_data['desc']
 
 
+def add_argument_kwargs(params, arg):
+    """
+    Build kwargs for parser.add_argument function
+    """
+    valid_hints = [int, float, str, bool]
+
+    if params[arg].annotation in valid_hints:
+        kwargs = {'type': params[arg].annotation}
+    else:
+        kwargs = {}
+
+    return kwargs
+
+
 def _add_args_from_callable(parser, callable_):
     """
     Modifies an args parser to include parameters from a callable, adding
@@ -164,20 +168,25 @@ def _add_args_from_callable(parser, callable_):
     Returns parsed args: required (list) and defaults (dict)
     """
     doc = _parse_doc(callable_.__doc__)
-    required, defaults = _parse_signature_from_callable(callable_)
+    required, defaults, params = _parse_signature_from_callable(callable_)
 
     for arg, default in defaults.items():
-        parser.add_argument('--' + arg, help=get_desc(doc, arg))
+        parser.add_argument('--' + arg,
+                            help=get_desc(doc, arg),
+                            **add_argument_kwargs(params, arg))
 
     for arg in required:
-        parser.add_argument(arg, help=get_desc(doc, arg))
+        parser.add_argument(arg,
+                            help=get_desc(doc, arg),
+                            **add_argument_kwargs(params, arg))
 
     return required, defaults
 
 
-def _process_file_entry_point(parser, entry_point, static_args):
+def _process_file_or_entry_point(parser, entry_point, static_args):
     """
-    Process a file entry point, returns the initialized dag and parsed args
+    Process a file entry point file or directory), returns the initialized dag
+    and parsed args
     """
     if Path('env.yaml').exists():
         env_dict = EnvDict('env.yaml')
@@ -189,37 +198,28 @@ def _process_file_entry_point(parser, entry_point, static_args):
         if args.log is not None:
             logging.basicConfig(level=args.log.upper())
 
-    with open(entry_point) as f:
-        dag_dict = yaml.load(f, Loader=yaml.SafeLoader)
-
-    if Path('env.yaml').exists():
-        env = EnvDict('env.yaml')
-        replaced = _args_to_replace_in_env(args, static_args)
-        env = env._replace_flatten_keys(replaced)
-        dag = DAGSpec(dag_dict, env=env).to_dag()
+    if Path(entry_point).is_dir():
+        dag = DAGSpec.from_directory(entry_point).to_dag()
     else:
-        dag = DAGSpec(dag_dict).to_dag()
+        with open(entry_point) as f:
+            dag_dict = yaml.load(f, Loader=yaml.SafeLoader)
+
+        if Path('env.yaml').exists():
+            env = EnvDict('env.yaml')
+            replaced = _args_to_replace_in_env(args, static_args)
+            env = env._replace_flatten_keys(replaced)
+            dag = DAGSpec(dag_dict, env=env).to_dag()
+        else:
+            dag = DAGSpec(dag_dict).to_dag()
 
     return dag, args
 
 
-def _process_factory_entry_point(parser, entry_point, static_args):
+def _process_factory_dotted_path(parser, dotted_path, static_args):
     """Parse a factory entry point, returns initialized dag and parsed args
+
     """
-    mod, name = _parse_module(entry_point)
-
-    try:
-        module = importlib.import_module(mod)
-    except ImportError as e:
-        raise ImportError('An error happened when trying to '
-                          'import module "{}"'.format(mod)) from e
-
-    try:
-        entry = getattr(module, name)
-    except AttributeError as e:
-        raise AttributeError('Could not get attribute "{}" from module '
-                             '"{}", make sure it is a valid callable'.format(
-                                 name, mod)) from e
+    entry = load_dotted_path(dotted_path, raise_=True)
 
     required, _ = _add_args_from_callable(parser, entry)
 
@@ -234,7 +234,7 @@ def _process_factory_entry_point(parser, entry_point, static_args):
         if args.log is not None:
             logging.basicConfig(level=args.log.upper())
 
-    # required by the function signature
+    # extract required (by using function signature) params from the cli args
     kwargs = {key: getattr(args, key) for key in required}
 
     # env and function defaults replaced
@@ -249,15 +249,31 @@ def _process_factory_entry_point(parser, entry_point, static_args):
 
 
 def _process_entry_point(parser, entry_point, static_args):
+    """Process an entry point from the user
+
+    Parameters
+    ----------
+    parser : CustomParser
+        The cli parser object
+
+    entry_point : str
+        An entry point string, this can be either path to a file or a dotted
+        path to a function that returns a DAG
     """
-    Process an entry point (either a file or a dotted path to a factory)
-    """
+    help_cmd = '--help' in sys.argv or '-h' in sys.argv
+
+    entry_file_exists = Path(entry_point).exists()
+    entry_obj = load_dotted_path(entry_point, raise_=False)
+
+    if (help_cmd and not entry_file_exists and not entry_obj):
+        args = parser.parse_args()
     # first check if the entry point is an existing file
-    if Path(entry_point).exists():
-        dag, args = _process_file_entry_point(parser, entry_point, static_args)
+    elif Path(entry_point).exists():
+        dag, args = _process_file_or_entry_point(parser, entry_point,
+                                                 static_args)
     # assume it's a dotted path to a factory
     else:
-        dag, args = _process_factory_entry_point(parser, entry_point,
+        dag, args = _process_factory_dotted_path(parser, entry_point,
                                                  static_args)
 
     return dag, args
