@@ -1,3 +1,30 @@
+"""
+
+On languages and kernels
+------------------------
+NotebookSource represents source code in a Jupyter notebook format (language
+agnostic). Apart from .ipynb, we also support any other extension supported
+by jupytext.
+
+Given a notebook, we have to know which language it is written in to extract
+upstream/product variables (though this only happens when the option of
+extracting dependencies automatically is on), we also have to determine the
+Jupyter kernel to use (this is always needed).
+
+The unequivocal place to store this information is in the notebook metadata
+section, but given that we advocate for the use of scripts (converted to
+notebooks via jupytext), they most likely won't contain metadata (metadata
+saving is turned off by default in jupytext), so we have to infer this
+ourselves.
+
+To make things more complex, jupytext adds its own metadata section but we are
+ignoring that for now.
+
+Given that there are many places where this information might be stored, we
+have a few rules to automatically determine language and kernel given a
+script/notebook.
+"""
+import ast
 from inspect import getargspec
 import tempfile
 from pathlib import Path
@@ -12,10 +39,9 @@ from ploomber.exceptions import RenderError, SourceInitializationError
 from ploomber.placeholders.Placeholder import Placeholder
 from ploomber.util import requires
 from ploomber.sources import Source
+from ploomber.static_analysis.extractors import extractor_class_for_language
 
 
-# TODO: make sure this works with R, Julia and other notebooks supported
-# by Jupyter
 class NotebookSource(Source):
     """
     A source object representing a jupyter notebook (or any format supported
@@ -37,10 +63,14 @@ class NotebookSource(Source):
     The render method prepares the notebook for execution: it adds the
     parameters and it makes sure kernelspec is defined
     """
-
-    @requires(['parso', 'pyflakes', 'jupytext', 'nbformat', 'papermill',
-               'jupyter_client'])
-    def __init__(self, primitive, hot_reload=False, ext_in=None,
+    @requires([
+        'parso', 'pyflakes', 'jupytext', 'nbformat', 'papermill',
+        'jupyter_client'
+    ])
+    def __init__(self,
+                 primitive,
+                 hot_reload=False,
+                 ext_in=None,
                  kernelspec_name=None,
                  static_analysis=False):
         # any non-py file must first be converted using jupytext, we need
@@ -62,8 +92,8 @@ class NotebookSource(Source):
             self._primitive = primitive.read_text()
         else:
             raise TypeError('Notebooks must be initialized from strings, '
-                            'Placeholder or pathlib.Path, got {}'
-                            .format(type(primitive)))
+                            'Placeholder or pathlib.Path, got {}'.format(
+                                type(primitive)))
 
         self.static_analysis = static_analysis
         self._kernelspec_name = kernelspec_name
@@ -86,17 +116,22 @@ class NotebookSource(Source):
         elif self._path is None and ext_in is not None:
             self._ext_in = ext_in
 
-        self._post_init_validation(str(self._primitive))
+        # try to determine language based on extension, though this test
+        # mught be inconclusive if dealing with a ipynb file
+        self._language = determine_language(self._ext_in)
 
         self._python_repr = None
         self._loc = None
         self._rendered_nb_str = None
         self._params = None
         self._nb_repr = None
+        self._nb_obj = None
 
         # this will raise an error if kernelspec_name is invalid
         self._get_nb_repr()
 
+        self._post_init_validation(str(self._primitive))
+
     @property
     def primitive(self):
         if self._hot_reload:
@@ -118,7 +153,9 @@ class NotebookSource(Source):
         # _get_nb_repr uses hot_reload, this ensures we always get the latest
         # version
         Path(tmp_in).write_text(self._get_nb_repr())
-        pm.execute_notebook(tmp_in, tmp_out, prepare_only=True,
+        pm.execute_notebook(tmp_in,
+                            tmp_out,
+                            prepare_only=True,
                             parameters=self._params)
 
         tmp_out = Path(tmp_out)
@@ -155,20 +192,27 @@ class NotebookSource(Source):
         source code passed, does not contain injected parameters.
 
         Adds kernelspec info if not present based on the kernelspec_name,
-        this metadata is required for papermill to know which kernel to use
+        this metadata is required for papermill to know which kernel to use.
+
+        An exception is raised if we cannot determine kernel information.
         """
         import nbformat
 
         # hot_reload causes to always re-evalaute the notebook representation
         if self._nb_repr is None or self._hot_reload:
-            if self._ext_in == 'ipynb':
-                self._nb_repr = self.primitive
-            else:
-                nb = _to_nb_obj(self.primitive,
-                                extension='py',
-                                kernelspec_name=self._kernelspec_name)
-                self._nb_repr = nbformat.writes(nb,
-                                                version=nbformat.NO_CONVERT)
+            # this is the notebook node representation
+            self._nb_obj = _to_nb_obj(
+                self.primitive,
+                # passing the underscored version
+                # because that's the only one available
+                # when this is initialized
+                language=self._language,
+                kernelspec_name=self._kernelspec_name)
+
+            # always write from nb_obj, even if this was initialized
+            # with a ipynb file, nb_obj contains kernelspec info
+            self._nb_repr = nbformat.writes(self._nb_obj,
+                                            version=nbformat.NO_CONVERT)
 
         return self._nb_repr
 
@@ -181,21 +225,33 @@ class NotebookSource(Source):
         # maybe we don't need to use pyflakes after all
         # we can also use compile. can pyflakes detect things that
         # compile cannot?
-        pass
+
+        params_cell, _ = find_cell_with_tag(self._nb_obj, 'parameters')
+
+        if params_cell is None:
+            loc = ' "{}"'.format(self.loc) if self.loc else ''
+            raise SourceInitializationError(
+                'Notebook{} does not have a cell tagged '
+                '"parameters"'.format(loc))
 
     def _post_render_validation(self, params):
         """
         Validate params passed against parameters in the notebook
         """
         if self.static_analysis:
-            import nbformat
-            # read the notebook with the injected parameters from the tmp
-            # location
-            nb_rendered = (nbformat
-                           .reads(self._rendered_nb_str,
-                                  as_version=nbformat.NO_CONVERT))
-            check_notebook(nb_rendered, params,
-                           filename=self._path or 'notebook')
+            if self.language == 'python':
+                import nbformat
+                # read the notebook with the injected parameters from the tmp
+                # location
+                nb_rendered = (nbformat.reads(self._rendered_nb_str,
+                                              as_version=nbformat.NO_CONVERT))
+                check_notebook(nb_rendered,
+                               params,
+                               filename=self._path or 'notebook')
+            else:
+                raise NotImplementedError(
+                    'static_analysis is only implemented for Python notebooks'
+                    ', set the option to False')
 
     @property
     def doc(self):
@@ -240,7 +296,46 @@ class NotebookSource(Source):
         # One approach is to use the ext if loaded from file, otherwise None
         return None
 
+    # FIXME: add this to the abstract class, probably get rid of "extension"
+    # since it's not informative (ipynb files can be Python, R, etc)
+    @property
+    def language(self):
+        """
+        Notebook Language (Python, R, etc), this is a best-effort property,
+        can be None if we could not determine the language
+        """
+        if self._language is None:
+            self._get_nb_repr()
 
+            try:
+                # make sure you return "r" instead of "R"
+                return self._nb_obj.metadata.kernelspec.language.lower()
+            except AttributeError:
+                return None
+
+        else:
+            return self._language
+
+    def _get_parameters_cell(self):
+        self._get_nb_repr()
+        cell, _ = find_cell_with_tag(self._nb_obj, tag='parameters')
+        return cell.source
+
+    # FIXME: A bit inefficient to initialize the extractor every time
+    def extract_upstream(self):
+        extractor_class = extractor_class_for_language(self.language)
+        return extractor_class(self._get_parameters_cell()).extract_upstream()
+
+    # FIXME: A bit inefficient to initialize the extractor every time
+    def extract_product(self):
+        extractor_class = extractor_class_for_language(self.language)
+        return extractor_class(self._get_parameters_cell()).extract_product()
+
+
+# FIXME: some of this only applies to Python notebooks (error about missing
+# parameters cells applies to every notebook), make sure the source takes
+# this into account, also check if there are any other functions that
+# are python specific
 def check_notebook(nb, params, filename):
     """
     Perform static analysis on a Jupyter notebook code cell sources
@@ -267,19 +362,15 @@ def check_notebook(nb, params, filename):
     # variable to collect all error messages
     error_message = '\n'
 
-    params_cell, _ = _find_cell_with_tag(nb, 'parameters')
+    params_cell, _ = find_cell_with_tag(nb, 'parameters')
 
-    if params_cell is None:
-        error_message += ('Notebook does not have a cell tagged '
-                          '"parameters"')
-    else:
-        # compare passed parameters with declared
-        # parameters. This will make our notebook behave more
-        # like a "function", if any parameter is passed but not
-        # declared, this will return an error message, if any parameter
-        # is declared but not passed, a warning is shown
-        res_params = compare_params(params_cell['source'], params)
-        error_message += res_params
+    # compare passed parameters with declared
+    # parameters. This will make our notebook behave more
+    # like a "function", if any parameter is passed but not
+    # declared, this will return an error message, if any parameter
+    # is declared but not passed, a warning is shown
+    res_params = compare_params(params_cell['source'], params)
+    error_message += res_params
 
     # run pyflakes and collect errors
     res = check_source(nb, filename=filename)
@@ -305,8 +396,10 @@ def json_serializable_params(params):
     params['product'] = params['product'].to_json_serializable()
 
     if params.get('upstream'):
-        params['upstream'] = {k: n.to_json_serializable() for k, n
-                              in params['upstream'].items()}
+        params['upstream'] = {
+            k: n.to_json_serializable()
+            for k, n in params['upstream'].items()
+        }
     return params
 
 
@@ -360,22 +453,31 @@ def check_source(nb, filename):
     err.seek(0)
 
     # return any error messages returned by pyflakes
-    return {'warnings': '\n'.join(warn.readlines()),
-            'errors': '\n'.join(err.readlines())}
+    return {
+        'warnings': '\n'.join(warn.readlines()),
+        'errors': '\n'.join(err.readlines())
+    }
 
 
-def _to_nb_obj(source, extension, kernelspec_name=None):
+def _to_nb_obj(source, language, kernelspec_name=None):
     """
-    Convert to jupyter notebook via jupytext, adding kernelspec details if
-    missing
+    Convert to jupyter notebook via jupytext, if the notebook does not contain
+    kernel information and the user did not pass a kernelspec_name explicitly,
+    we will try to infer the language and select a kernel appropriately.
+
+    If a valid kernel is found, it is added to the notebook. If none of this
+    works, an exception is raised.
+
+    If also converts the code string to its notebook node representation,
+    adding kernel data accordingly.
 
     Parameters
     ----------
     source : str
         Jupyter notebook (or jupytext compatible formatted) document
 
-    extension : str
-        Document format
+    language : str
+        Programming language
 
     Returns
     -------
@@ -393,30 +495,63 @@ def _to_nb_obj(source, extension, kernelspec_name=None):
     """
     import jupytext
     import jupyter_client
-    # NOTE: how is this different to just doing fmt='.py'
-    nb = jupytext.reads(source, fmt={'extension': '.'+extension})
-    nb_kernelspec = nb.metadata.get('kernelspec')
 
-    if nb_kernelspec is None and kernelspec_name is None:
+    # let jupytext figure out the format
+    nb = jupytext.reads(source, fmt=None)
+
+    kernel_name = determine_kernel_name(nb, language, kernelspec_name)
+
+    # cannot keep going if we don't have the kernel name
+    if kernel_name is None:
         raise SourceInitializationError(
-            'Notebook does not contains kernelspec metadata and '
+            'Notebook does not contain kernelspec metadata and '
             'kernelspec_name was not specified, either add '
             'kernelspec info to your source file or specify '
-            'a kernelspec by name')
+            'a kernelspec by name. To see list of installed kernels run '
+            '"jupyter kernelspec list" in the terminal (first column '
+            'indicates the name). Python is usually named "python3", '
+            'R usually "ir"')
 
-    # only use kernelspec_name when there is no kernelspec info in the nb
-    if nb_kernelspec is None and kernelspec_name is not None:
-        k = jupyter_client.kernelspec.get_kernel_spec(kernelspec_name)
+    kernelspec = jupyter_client.kernelspec.get_kernel_spec(kernel_name)
 
-        nb.metadata.kernelspec = {
-            "display_name": k.display_name,
-            "language": k.language,
-            "name": kernelspec_name
-        }
+    nb.metadata.kernelspec = {
+        "display_name": kernelspec.display_name,
+        "language": kernelspec.language,
+        "name": kernel_name
+    }
 
     return nb
 
 
+def determine_kernel_name(nb, language, kernelspec_name):
+    """
+    Try to determine kernel name, first check notebook metadata, then
+    check if the user explicitly provided a kernelspec_name. If None of this
+    works use the language info, which should not be empty by the time this
+    is executed
+    """
+    try:
+        return nb.metadata.kernelspec.name
+    except AttributeError:
+        pass
+
+    if kernelspec_name is not None:
+        return kernelspec_name
+
+    # two most common cases
+    lang2kernel = {'python': 'python3', 'r': 'ir'}
+
+    if language in lang2kernel:
+        return lang2kernel[language]
+
+    is_python_ = is_python(nb)
+
+    if is_python_:
+        return 'python3'
+    else:
+        return None
+
+
 def inject_cell(model, params):
     """Inject params (by adding a new cell) to a model
 
@@ -456,19 +591,22 @@ def inject_cell(model, params):
     else:
         kwargs = {}
 
-    model['content'] = parameterize_notebook(nb, params,
+    model['content'] = parameterize_notebook(nb,
+                                             params,
                                              report_mode=False,
                                              **kwargs)
 
 
+# FIXME: this is used in the task itself in the .develop() feature, maybe
+# move there?
 def _cleanup_rendered_nb(nb):
-    cell, i = _find_cell_with_tag(nb, 'injected-parameters')
+    cell, i = find_cell_with_tag(nb, 'injected-parameters')
 
     if i is not None:
         print('Removing injected-parameters cell...')
         nb['cells'].pop(i)
 
-    cell, i = _find_cell_with_tag(nb, 'debugging-settings')
+    cell, i = find_cell_with_tag(nb, 'debugging-settings')
 
     if i is not None:
         print('Removing debugging-settings cell...')
@@ -484,7 +622,46 @@ def _cleanup_rendered_nb(nb):
     return nb
 
 
-def _find_cell_with_tag(nb, tag):
+def is_python(nb):
+    """
+    Determine if the notebook is Python code for a given notebook object, look
+    for metadata.kernelspec.language first, if not defined, try to guess if
+    it's Python, it's conservative and it returns False if the code is valid
+    Python but contains (<-), in which case it's much more likely to be R
+    """
+    is_python_ = None
+
+    # check metadata first
+    try:
+        language = nb.metadata.kernelspec.language
+    except AttributeError:
+        pass
+    else:
+        is_python_ = language == 'python'
+
+    # no language defined in metadata, check if it's valid python
+    if is_python_ is None:
+        code_str = '\n'.join([c.source for c in nb.cells])
+
+        try:
+            ast.parse(code_str)
+        except SyntaxError:
+            is_python_ = False
+        else:
+            # there is a lot of R code which is also valid Python code! So let's
+            # run a quick test. It is very unlikely to have "<-" in Python (
+            # {less than} {negative} but extremely common {assignment}
+            if '<-' not in code_str:
+                is_python_ = True
+
+    # inconclusive test...
+    if is_python_ is None:
+        is_python_ = False
+
+    return is_python_
+
+
+def find_cell_with_tag(nb, tag):
     """
     Find a cell with a given tag, returns a cell, index tuple. Otherwise
     (None, None)
@@ -496,3 +673,18 @@ def _find_cell_with_tag(nb, tag):
                 return c, i
 
     return None, None
+
+
+def determine_language(extension):
+    """
+    A function to determine programming language given file extension,
+    returns programming language name (all lowercase) if could be determined,
+    None if the test is inconclusive
+    """
+    if extension.startswith('.'):
+        extension = extension[1:]
+
+    mapping = {'py': 'python', 'r': 'r', 'R': 'r'}
+
+    # ipynb can be many languages, it must return None
+    return mapping.get(extension)
