@@ -6,7 +6,6 @@ from ploomber.products import Product
 from ploomber.placeholders.Placeholder import Placeholder
 from ploomber.exceptions import SourceInitializationError
 from ploomber.sql import infer
-from ploomber.util import util
 from ploomber import static_analysis
 
 
@@ -199,6 +198,14 @@ class SQLSourceMixin:
     def extension(self):
         return 'sql'
 
+    def extract_product(self):
+        return static_analysis.sql.SQLExtractor(
+            self._placeholder._raw).extract_product()
+
+    def extract_upstream(self):
+        return static_analysis.sql.SQLExtractor(
+            self._placeholder._raw).extract_upstream()
+
 
 class SQLScriptSource(SQLSourceMixin, PlaceholderSource):
     """
@@ -270,9 +277,10 @@ class SQLScriptSource(SQLSourceMixin, PlaceholderSource):
             #                       .format(infered_relations, actual_len))
 
     def render(self, params):
-        extracted = (static_analysis.sql.extract_product_from_sql(
-            self._placeholder._raw))
-
+        # FIXME: inefficient, initialize once and only update if needed
+        # (i.e. hot reload is on)
+        extracted = static_analysis.sql.SQLExtractor(
+            self._placeholder._raw).extract_product()
         # the code itself might already define the product, no need to pass it
         # TODO: verify that the product passed and the one defined are the same
         if extracted:
