@@ -15,17 +15,20 @@ import yaml
 import logging
 from pathlib import Path
 from collections.abc import MutableMapping
+from glob import iglob
+from itertools import chain
 import pprint
 
 from ploomber import products
 from ploomber import DAG, tasks
-from ploomber.util.util import load_dotted_path, find_file_recursively
-from ploomber.static_analysis import project
-from ploomber.spec.TaskSpec import TaskSpec
+from ploomber.util.util import (load_dotted_path, find_file_recursively,
+                                call_with_dictionary)
+from ploomber.spec.TaskSpec import TaskSpec, suffix2taskclass
 from ploomber.spec import validate
 from ploomber.dag.DAGConfiguration import DAGConfiguration
 from ploomber.exceptions import DAGSpecInitializationError
 from ploomber.env.expand import expand_raw_dictionary
+from ploomber.tasks import NotebookRunner
 
 logger = logging.getLogger(__name__)
 pp = pprint.PrettyPrinter(indent=4)
@@ -109,47 +112,7 @@ class DAGSpec(MutableMapping):
         if 'meta' not in self.data:
             self.data['meta'] = {}
 
-        self.default_meta(self.data['meta'])
-
-    @classmethod
-    def default_meta(cls, meta=None):
-        """Fill missing values in a meta dictionary
-        """
-        if meta is None:
-            meta = {}
-
-        valid = {
-            'extract_upstream', 'extract_product', 'product_default_class',
-            'product_relative_to_source', 'jupyter_hot_reload'
-        }
-        validate.keys(valid, meta, name='dag spec')
-
-        if 'extract_upstream' not in meta:
-            meta['extract_upstream'] = True
-
-        if 'extract_product' not in meta:
-            meta['extract_product'] = True
-
-        if 'product_relative_to_source' not in meta:
-            meta['product_relative_to_source'] = False
-
-        if 'jupyter_hot_reload' not in meta:
-            meta['jupyter_hot_reload'] = False
-
-        defaults = {
-            'SQLDump': 'File',
-            'NotebookRunner': 'File',
-            'SQLScript': 'SQLRelation'
-        }
-
-        if 'product_default_class' not in meta:
-            meta['product_default_class'] = defaults
-        else:
-            for class_, prod in defaults.items():
-                if class_ not in meta['product_default_class']:
-                    meta['product_default_class'][class_] = prod
-
-        return meta
+        self.data['meta'] = Meta.default_meta(self.data['meta'])
 
     def __getitem__(self, key):
         return self.data[key]
@@ -249,6 +212,29 @@ class DAGSpec(MutableMapping):
                                              'pipeline.yaml')
             raise exc from e
 
+    @classmethod
+    def from_directory(cls, path_to_dir):
+        """Construct a DAGSpec from a directory
+
+        Look for scripts (``.py``, ``.R`` or ``.ipynb``) in the directory and
+        interpret them as task sources, file names are assigned as task names
+        (without extension). The spec is generated with the default values in
+        the "meta" section.
+        """
+        extensions = [
+            name for name, class_ in suffix2taskclass.items()
+            if class_ is NotebookRunner
+        ]
+
+        files = chain.from_iterable(iglob('*' + ext) for ext in extensions)
+
+        tasks = [{
+            'source': file_,
+            'name': str(Path(file_).with_suffix(''))
+        } for file_ in files]
+
+        return cls({'tasks': tasks})
+
     @classmethod
     def from_file(cls, path):
         """
@@ -262,35 +248,92 @@ class DAGSpec(MutableMapping):
         return spec
 
 
+class Meta:
+    """Schema for meta section in pipeline.yaml
+    """
+    @classmethod
+    def default_meta(cls, meta=None):
+        """Fill missing values in a meta dictionary
+        """
+        if meta is None:
+            meta = {}
+
+        valid = {
+            'extract_upstream', 'extract_product', 'product_default_class',
+            'product_relative_to_source', 'jupyter_hot_reload'
+        }
+        validate.keys(valid, meta, name='dag spec')
+
+        if 'extract_upstream' not in meta:
+            meta['extract_upstream'] = True
+
+        if 'extract_product' not in meta:
+            meta['extract_product'] = True
+
+        if 'product_relative_to_source' not in meta:
+            meta['product_relative_to_source'] = False
+
+        if 'jupyter_hot_reload' not in meta:
+            meta['jupyter_hot_reload'] = False
+
+        defaults = {
+            'SQLDump': 'File',
+            'NotebookRunner': 'File',
+            'SQLScript': 'SQLRelation'
+        }
+
+        if 'product_default_class' not in meta:
+            meta['product_default_class'] = defaults
+        else:
+            for class_, prod in defaults.items():
+                if class_ not in meta['product_default_class']:
+                    meta['product_default_class'][class_] = prod
+
+        return meta
+
+
 def process_tasks(dag, tasks, dag_spec, root_path=None):
+    """
+    Initialize Task objects from TaskSpec, extract product and dependencies
+    if needed and set the dag dependencies structure
+    """
     root_path = root_path or '.'
 
-    # determine if we need to run static analysis
-    meta = dag_spec['meta']
-    sources = [task_dict['source'] for task_dict in tasks]
-    extracted = project.infer_from_path(root_path,
-                                        templates=sources,
-                                        upstream=meta['extract_upstream'],
-                                        product=meta['extract_product'])
-
     upstream = {}
+    source_obj = {}
+    extract_up = dag_spec['meta']['extract_upstream']
+    extract_prod = dag_spec['meta']['extract_product']
 
+    # first pass: init tasks and them to dag
     for task_dict in tasks:
-        source = task_dict['source']
+        fn = task_dict['class']._init_source
+        kwargs = {'kwargs': {}, **task_dict}
+        source = call_with_dictionary(fn, kwargs=kwargs)
 
-        if dag_spec['meta']['extract_upstream']:
-            task_dict['upstream'] = extracted['upstream'][source]
-
-        if dag_spec['meta']['extract_product']:
-            task_dict['product'] = extracted['product'][source]
+        if extract_prod:
+            task_dict['product'] = source.extract_product()
 
         task, up = task_dict.to_task(dag, root_path)
-        upstream[task] = up
 
-    # once we added all tasks, set upstream dependencies
+        if extract_prod:
+            logger.debug('Extracted product for task "%s": %s', task.name,
+                         task.product)
+
+        upstream[task] = up
+        source_obj[task] = source
+
+    # second optional pass: extract upstream
+    if extract_up:
+        for task in list(dag.values()):
+            upstream[task] = task.source.extract_upstream()
+            logger.debug('Extracted upstream dependencies for task %s: %s',
+                         task.name, upstream[task])
+
+    # Last pass: set upstream dependencies
     for task in list(dag.values()):
-        for task_dep in upstream[task]:
-            task.set_upstream(dag[task_dep])
+        if upstream[task]:
+            for task_name in upstream[task]:
+                task.set_upstream(dag[task_name])
 
 
 def init_clients(dag, clients):
