@@ -1,6 +1,7 @@
 """
 Create Tasks from dictionaries
 """
+from copy import copy
 from pathlib import Path
 from collections.abc import MutableMapping, Iterable, Mapping
 
@@ -10,6 +11,8 @@ from ploomber.spec import validate
 
 suffix2taskclass = {
     '.py': tasks.NotebookRunner,
+    '.R': tasks.NotebookRunner,
+    '.r': tasks.NotebookRunner,
     '.ipynb': tasks.NotebookRunner,
     '.sql': tasks.SQLScript,
     '.sh': tasks.ShellScript
@@ -23,6 +26,11 @@ class TaskSpec(MutableMapping):
         self.meta = meta
         self.validate()
 
+        # initialie required elements
+        self.data['class'] = get_class_obj(self.data)
+        # in task specs, we assume source is always a path to a file
+        self.data['source'] = Path(self.data['source'])
+
     def validate(self):
         if 'upstream' not in self.data:
             self.data['upstream'] = None
@@ -52,11 +60,11 @@ class TaskSpec(MutableMapping):
     def to_task(self, dag, root_path):
         """Returns a Task instance
         """
-        task_dict = self.data
+        task_dict = copy(self.data)
         upstream = _make_iterable(task_dict.pop('upstream'))
-        class_ = get_task_class(task_dict)
+        class_ = task_dict.pop('class')
 
-        product = _init_product(task_dict, self.meta, class_, root_path)
+        product = init_product(task_dict, self.meta, class_, root_path)
 
         _init_client(task_dict)
 
@@ -67,9 +75,9 @@ class TaskSpec(MutableMapping):
         on_render = task_dict.pop('on_render', None)
         on_failure = task_dict.pop('on_failure', None)
 
-        task = class_(source=Path(source_raw),
+        task = class_(source=source_raw,
                       product=product,
-                      name=name_raw or source_raw,
+                      name=name_raw or str(source_raw),
                       dag=dag,
                       **task_dict)
 
@@ -101,18 +109,41 @@ class TaskSpec(MutableMapping):
         return len(self.data)
 
 
-def _make_iterable(o):
-    if isinstance(o, Iterable) and not isinstance(o, str):
-        return o
-    elif o is None:
-        return []
+def get_class_obj(task_spec):
+    """
+    Returns the class for the TaskSpec, if the spec already has the class
+    name (str), it just returns the actual class object with such name,
+    otherwise it tries to guess based on the file extension
+
+    Task class is determined by the 'class' key, if missing. Defaults
+    are used by inspecting the 'source' key: NoteboonRunner (.py),
+    SQLScript (.sql) and BashScript (.sh).
+    """
+    class_name = task_spec.get('class', None)
+
+    if class_name:
+        class_ = getattr(tasks, class_name)
     else:
-        return [o]
+        suffix = Path(task_spec['source']).suffix
+
+        if suffix2taskclass.get(suffix):
+            class_ = suffix2taskclass[suffix]
+        else:
+            raise KeyError('No default task class available for task with '
+                           'source: '
+                           '"{}". Default class is only available for '
+                           'files with extensions {}. '
+                           'Set an explicit class key'.format(
+                               task_spec['source'], set(suffix2taskclass)))
+
+    return class_
 
 
 # FIXME: how do we make a default product client? use the task's client?
-def _init_product(task_dict, meta, task_class, root_path):
+def init_product(task_dict, meta, task_class, root_path):
     """
+    Initialize product.
+
     Resolution logic order:
         task.product_class
         meta.{task_class}.product_default_class
@@ -160,6 +191,15 @@ def _init_product(task_dict, meta, task_class, root_path):
                      **kwargs)
 
 
+def _make_iterable(o):
+    if isinstance(o, Iterable) and not isinstance(o, str):
+        return o
+    elif o is None:
+        return []
+    else:
+        return [o]
+
+
 def resolve_product(product_raw, relative_to, class_):
     if class_ != products.File:
         return product_raw
@@ -185,34 +225,6 @@ def _init_client(task_dict):
         task_dict['client'] = load_dotted_path(dotted_path)()
 
 
-def get_task_class(task_dict):
-    """
-    Pops 'class' key if it exists
-
-    Task class is determined by the 'class' key, if missing. Defaults
-    are used by inspecting the 'source' key: NoteboonRunner (.py),
-    SQLScript (.sql) and BashScript (.sh).
-    """
-    class_name = task_dict.pop('class', None)
-
-    if class_name:
-        class_ = getattr(tasks, class_name)
-    else:
-        suffix = Path(task_dict['source']).suffix
-
-        if suffix2taskclass.get(suffix):
-            class_ = suffix2taskclass[suffix]
-        else:
-            raise KeyError('No default task class available for task with '
-                           'source: '
-                           '"{}". Default class is only available for '
-                           'files with extensions {}, otherwise you should '
-                           'set an explicit class key'.format(
-                               task_dict['source'], set(suffix2taskclass)))
-
-    return class_
-
-
 def get_value_at(d, dotted_path):
     current = d
 
