@@ -50,7 +50,18 @@ class NotebookConverter:
                  nbconvert_export_kwargs=None):
         if exporter_name is None:
             #  try to infer it from the extension
-            exporter_name = Path(path_to_output).suffix[1:]
+            suffix = Path(path_to_output).suffix
+
+            if not suffix:
+                raise ValueError('Could not determine output format for '
+                                 'product: "{}" because it has no extension. '
+                                 'Either add an extension '
+                                 'or explicitly pass a '
+                                 '"nbconvert_exporter_name" to the task '
+                                 'constructor'.format(path_to_output))
+
+            exporter_name = suffix[1:]
+
 
         self.exporter = self._get_exporter(exporter_name)
         self.path_to_output = path_to_output
@@ -112,7 +123,7 @@ class NotebookConverter:
         elif isinstance(content, bytes):
             path.write_bytes(content)
         else:
-            raise TypeError('nbconvert returned a converted notbeook with'
+            raise TypeError('nbconvert returned a converted notebook with'
                             'unknown format, only text and binary objects '
                             'are supported')
 
@@ -143,9 +154,10 @@ class NotebookRunner(Task):
     papermill_params : dict, optional
         Other parameters passed to papermill.execute_notebook, defaults to None
     kernelspec_name: str, optional
-        Kernelspec name to use, if the notebook already includes kernelspec
-        data (in metadata.kernelspec), this is ignored, otherwise, the kernel
-        is looked up using the jupyter_client.kernelspec.get_kernel_spec
+        Kernelspec name to use, if the file extension provides with enough
+        information to choose a kernel or the notebook already includes
+        kernelspec data (in metadata.kernelspec), this is ignored, otherwise,
+        the kernel is looked up using jupyter_client.kernelspec.get_kernel_spec
     nbconvert_exporter_name: str, optional
         Once the notebook is run, this parameter controls whether to export
         the notebook to a different parameter using the nbconvert package,
@@ -188,7 +200,7 @@ class NotebookRunner(Task):
                  name=None,
                  params=None,
                  papermill_params=None,
-                 kernelspec_name='python3',
+                 kernelspec_name=None,
                  nbconvert_exporter_name=None,
                  ext_in=None,
                  nb_product_key='nb',
@@ -203,13 +215,18 @@ class NotebookRunner(Task):
         self.nb_product_key = nb_product_key
         self.static_analysis = static_analysis
         self.local_execution = local_execution
-        super().__init__(source, product, dag, name, params)
 
         if 'cwd' in self.papermill_params and self.local_execution:
             raise KeyError('If local_execution is set to True, "cwd" should '
                            'not appear in papermill_params, as such '
                            'parameter will be set by the task itself')
 
+        kwargs = dict(hot_reload=dag._params.hot_reload)
+        self._source = NotebookRunner._init_source(source, kwargs, ext_in,
+                                                   kernelspec_name,
+                                                   static_analysis)
+        super().__init__(product, dag, name, params)
+
         if isinstance(self.product, MetaProduct):
             if self.product.get(nb_product_key) is None:
                 raise KeyError('Key "{}" does not exist in product: {}. '
@@ -229,11 +246,16 @@ class NotebookRunner(Task):
                                             nbconvert_exporter_name,
                                             nbconvert_export_kwargs)
 
-    def _init_source(self, source, kwargs):
+    @staticmethod
+    def _init_source(source,
+                     kwargs,
+                     ext_in=None,
+                     kernelspec_name=None,
+                     static_analysis=False):
         return NotebookSource(source,
-                              ext_in=self.ext_in,
-                              kernelspec_name=self.kernelspec_name,
-                              static_analysis=self.static_analysis,
+                              ext_in=ext_in,
+                              kernelspec_name=kernelspec_name,
+                              static_analysis=static_analysis,
                               **kwargs)
 
     def develop(self):
